/* PIC-Brenner L = gro */

/*
Unterschiede zur K Version:
- separates Netzteil, daher keine USB-Belastung
- VPP wird nicht invers, sondern proportional eingestellt

*/

#include "StdTypes.h"
#include "startup.h"
#include "STM32F103all.h"
#include "config.h"
#include "systick.h"
#include "conv.h"
#include "serial.h"
#include "usb.h"
#include "gio.h"
#include "brenner.h"

dword Akku;
dword SchreibDatenMerker;
dword LeseDaten;
long  VCCWert;
long  VPPWert;
int   KommandoLenge;
int   Datenlenge;
bool  Kommentar;
bool  ObBrenner;
bool  echo;
int   warteClkHi;
int   warteClkLo;


const char BekannteKommandos[] = 
 {'E','G','K','L','M','P','Q','S','U','V','W','X','Y','Z',0};


void Meldung (word wo)
{ String_Out("(STM32PicProg v0.1) ", wo); 
  Char_Out('<', wo);
  String_Out((char*)&BekannteKommandos, wo);
  Char_Out('>', wo);
  CRLF_Out(wo);
}

/*
Pinbelegung PicProgK:
MCLR/VPP low  ber PA0
MCLR/VPP high ber PA1
VCC      ein/aus ber PA4
CLOCK    ber PB10
DATA-Out ber PB11
DATA-In  ber PB12

VCC variabel ber TIM4_CH3,
Vpp variabel ber TIM4_CH4
*/

// Primitive zur Controller-Pin-Ansteuerung
// 16..31 = reset, 0..15 = set
void  Clock_hi (void) { GPIOB_BSRR = 1<<(10); }
void  Clock_lo (void) { GPIOB_BSRR = 1<<(16+10); }

void  Data_hi  (void) { GPIOB_BSRR = 1<<(11); }
void  Data_lo  (void) { GPIOB_BSRR = 1<<(16+11); }

// Ansteuerung der Versorgungsspannungen
// 16..31 = reset, 0..15 = set, 
// die Brenner-HW dreht das um
void  VCC_hi   (void) { GPIOA_BSRR = 1<<(16+4); }
void  VCC_lo   (void) { GPIOA_BSRR = 1<<(4); }

void  VPP_hi   (void) { GPIOA_BSRR = 1<<16; // low Treiber aus
                        GPIOA_BSRR = 1<<17; // high Treiber ein
		      }
void  VPP_lo   (void) { GPIOA_BSRR = 2;  // high Treiber aus
                        GPIOA_BSRR = 1;  // low Treiber ein
		      } 	
void  VPP_hz   (void) { GPIOA_BSRR = (1<<16) | 2; }

// MCLR / VPP ansteuern
void  SetzeVPP (void)
{ switch (Akku)
  { case 1:  VPP_hz(); break;
    case 2:  VPP_hi(); break;
    default: VPP_lo(); break;
  }
}


// Lese-Datenbit hereinholen
bool  GetDataBit (void) 
{ dword D;
  D = GPIOB_IDR & (1<<12);
  if (D) return true;
  return false;
}

// fr Testzwecke
const char hwtxt[] = "Clock:x Data:x Read:x Vcc:? Vpp:?";
void HWStatusAusgabe(word wo)
{ char txt[35];
  int i;
  i = 0;
  while (i<35) {txt[i] = hwtxt[i]; i++;}
  txt[34] = 0;
  if (GPIOB_IDR & (1<<10)) txt[6]  = 'H'; else txt[6]  = 'L';
  if (GPIOB_IDR & (1<<11)) txt[13] = 'H'; else txt[13] = 'L';
  if (GetDataBit())       txt[20] = 'H'; else txt[20] = 'L';
  if (GPIOB_IDR & (1<<8)) txt[26] = 'H'; else txt[26] = 'L';
  if (GPIOB_IDR & (1<<9)) txt[32] = 'H'; else txt[32] = 'L';
  String_Out(txt, wo);
  CRLF_Out(wo);
  Akku = 0;
}

// fr Testzwecke
void SetzeClkData (byte B)
{ if (B & 1) Clock_hi(); else Clock_lo();
  if (B & 2) Data_hi(); else Data_lo();
}

void SchalteLedEin (void)
{ GPIOB_BSRR = 1<<15; }

void SchalteLedAus (void)
{ GPIOB_BSRR = 1UL<<(16+15); }


// VPP in Millivolts
// hier proportional.
// PWM von 1 bis 1999
void SetzeVoltVPP (void)
{ long L;
  L = Akku;
  if (L > 14000) L = 14000;
  if (L < 6000)  L = 6000;
  VPPWert = L;
  L = L * 133;
  L = L / 1024;
  // L min = 796
  // L max = 1859
  TIM4_CCR4 = L;  
}

// Versorgungsspannung ansteuern
void SetzeVCC (void)
{ if (Akku==1)
    VCC_hi();
  else
    VCC_lo();
}

// VCC in Millivolts
// hier subtraktiv Uref=1.23V
// PWM von 1 bis 1999
void SetzeVoltVCC (void)
{ long L;

  L = Akku;
  if (L > 5500) L = 5500;
  if (L < 2000) L = 2000;
  VCCWert = L;
  L = 5500 - L;
  if (L<0) L = 0;
  L = L * 9;
  L = L >> 4;

  TIM4_CCR3 = L;  
}


// simple Trampelschleife
void Wartemal (long L) { while(L) --L; }

// Kurzwartezeiten setzen
void SetzeTiming (void)
{ dword i;
  i =  Akku;
  if (i<5)   i = 5;
  if (i>500) i = 500;
  warteClkHi = i;
  warteClkLo = i;
}
  

// alle Daten werden auf die fallende Flanke bernommen

void KommandoZumPIC (void)
{ int i;
  dword D;

  i = 0;
  D = Akku;
  while (i < KommandoLenge)
  { Clock_hi();
    if (D&1)
       Data_hi();
    else
       Data_lo();
    D = D >> 1;   
    Wartemal(warteClkHi);
    Clock_lo();
    Wartemal(warteClkLo);
    ++i;   
  }  
  Wartemal(warteClkHi+warteClkLo);
}

void SchreibeWort (void)
{ int i;
  dword D;

  i = 0;
  D = Akku;
  while (i < Datenlenge)
  { Clock_hi();
    if (D&1)
       Data_hi();
    else
       Data_lo();
    D = D >> 1;   
    Wartemal(warteClkHi);
    Clock_lo();
    Wartemal(warteClkLo);
    ++i;   
  }  
}

// Bitwechsel bei Clock L->H, Auslesen um und bei H->L
dword LiesAus (void)
{ int i;
  dword D, M;
  i = 0;
  D = 0;
  M = 1;
  Data_hi();  // oder Tristate
  while (i < Datenlenge)
  { Clock_hi();
    Wartemal(warteClkHi);
    if (GetDataBit()) D = D | M;
    Clock_lo();
    Wartemal(warteClkLo);
    ++i;   
    M = M<<1;
  }  
  return D;
}

void Auslesen (word wo)
{ dword D;
  D = LiesAus();
  Data_lo();
  Char_Out('[', wo);
  Dezi_Out(D,1,wo);
  Char_Out(']', wo);
}


void BrennerTalk (word wo)
{ char c;

  if (!RxAvail(wo)) return;
  
  c =  GetChar(wo); if (!c) return;

  if (echo) Char_Out(c, wo);
  
  if (Kommentar)  
  { Char_Out(c, wo);
    if (c==')') Kommentar = false;
    return;
  }

  if (c=='(')
  { Char_Out(c, wo);
    Kommentar = true;
    return;
  }      
  
  if ((c>='0') && (c<='9'))
  { c = c - '0';
    Akku = Akku * 10 + c;
    return;
  }  

  switch (c)
  { case 'A':  break;
    case 'B':  break;
    case 'C':  break;
    case 'D':  break;
    case 'E':  ObBrenner = false; 
               String_Out(Manifest, wo);
	       String_Out("\r\n> ", wo);
               return;
    case 'F':                        break;
    case 'G':  Meldung(wo);          break;
    case 'H':                        break;
    case 'I':                        break;
    case 'J':                        break;
    case 'K':  KommandoZumPIC();     break;
    case 'L':  Auslesen(wo);         break;
    case 'M':  SchreibDatenMerker = Akku; break;
    case 'N':                        break;
    case 'O':                        break;
    case 'P':  SetzeVPP();           break;
    case 'Q':  SetzeVoltVPP();       break;
    case 'R':                        break;
    case 'S':  SchreibeWort();       break;
    case 'T':  SetzeTiming();        break;
    case 'U':  SetzeVCC();           break;
    case 'V':  SetzeVoltVCC();       break;
    case 'W':  Warten(wo);           break;
    case 'X':  Datenlenge = Akku;    break;
    case 'Y':  KommandoLenge = Akku; break;
    case 'Z':  BrennerInit();        break;   
    
    // Sonderzeug
    case ';':  Char_Out(';', wo);    break; 
    case '*':  Char_Out('*', wo);    break; 
    case '#':  Char_Out('#', wo);    break; 
    case '!':  SetzeClkData(Akku);   break;
    case '?':  HWStatusAusgabe(wo);  break;
    case '<':  SchalteLedEin();      break;
    case '>':  SchalteLedAus();      break;
    case '/':  echo = !echo; break;
    default: return;
  }
  Akku = 0;
}


// fr das Abmessen von Zeiten wird TIM2 benutzt
// der Counter und der Vorteiler haben beide je nur 16 Bit
void TimerAufsetzen (void)
{ TIM2_CR1   = 0;            // gestoppt
  TIM2_CR2   = 0;
  TIM2_SMCR  = 0;            // kein Slavemode
  TIM2_SR    = 0;            // alle pending Ints aus
  TIM2_DIER  = 0;            // kein Interrupt
  TIM2_CCER  = 0;            // Capture/Comp erstmal aus
  TIM2_CCMR1 = 0;            // kein PWM
  TIM2_CCMR2 = 0;
  TIM2_CNT   = 0;           
  TIM2_PSC   = F_Platform/10000; // 0.1 ms Takt
  TIM2_ARR   = 0xFFFF;      
  TIM2_CCR1  = 0;           
  TIM2_CCER  = 0;           
  TIM2_CR1   = 1;            // starten
  TIM2_EGR   = 1;
}

void PWMAufsetzen (void)
{ TIM4_CR1   = 0;            // gestoppt
  TIM4_CR2   = 0;
  TIM4_SMCR  = 0;            // kein Slavemode
  TIM4_SR    = 0;            // alle pending Ints aus
  TIM4_DIER  = 0;            // kein Interrupt
  TIM4_CCER  = 0;            // Capture/Comp erstmal aus
  TIM4_CCER  = 0;            // erstmal
  TIM4_CCMR1 = 0;
  TIM4_CCMR2 = (6<<12) |     // PWM mode 1 fr ch4
	       (1<<11) |     // ch4 preload
               (6<<4)  |     // PWM mode 1 fr ch3
	       (1<<3)  |     // ch3 preload
	       0;
  
  TIM4_CNT   = 0;           
  TIM4_PSC   = 1;     
  TIM4_ARR   = 2000;         // PWM von 1 bis 1999
  TIM4_CCR1  = 0;           
  TIM4_CCR3  = 1000;         // PWM erstmal in Mitte  
  TIM4_CCR4  = 1000;         // dito.  
  TIM4_CCER  = (1<<12) |     // enable ch4
               (1<<8)  |     // enable ch3
	       0;
  TIM4_CR1   = 1;            // starten
  TIM4_EGR   = 1;
}

// Warten in 0.1 MilliSekunden
void Warten (word wo)
{ dword w;
  w = Akku;
  if (!w) return;
  if (w>65000) w = 65000;
  TIM2_CNT = 0;
  while (TIM2_CNT < w);
}


void BrennerInit (void)
{ VPP_lo();
  VCC_lo();
  PWMAufsetzen();
  KommandoLenge = 6;
  Datenlenge    = 16;
  Kommentar = false;
  Data_lo();
  Clock_lo();
  TimerAufsetzen();
  warteClkHi = 10;
  warteClkLo = 10;

  Akku = 13000;
  SetzeVoltVPP();
  Akku = 5000;
  SetzeVoltVCC();
  Akku = 0;
}

